/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

/**
* Funktion, die die Datenbanktabellen und die Level erstellt
* @return { tables: "Array mit Datenbanktabellen", levels: "Array mit Leveldaten" }
*/
function getHumanDbmsLevels(
    fktRandomInt    // Funktion mit Paramtern min und max, die pseudozufällige aber wiedererzeugbare Folge von Ganzzahlen aus [min; max] liefert
) {
    const result = {
        tabletype: 'Ergebnistabelle',
        tables: [
            {
                name: 'PUNKT',
                csv: 'name;x;y\nA;7;4\nB;5;4\nC;7;3\nP;2;3'
            },
            {
                name: 'STRECKE',
                csv: 'name;x1;y1;x2;y2\na;5;4;7;3\nb;7;4;7;3\nc;7;4;5;4'
            }
        ],
        levels: [
            {
                task: "<p>Willkommen bei deinem neuen Job als Datenbankmanagementsystem! Du sollst zu eingehenden Abfragen Ergebnistabellen erstellen und versenden.</p><p>Mache dich zunächst mit den vorhandenen Datenbanktabellen vertraut: Dank des Auswahlfeldes kannst du die sichtbare Datenbanktabelle auswählen.</p><p>Klicke auf „Weiter“, wenn du dir beide Datenbanktabellen angesehen hast.</p>",
                locked: false
            },
            {
                task: "<p>Aus wie vielen Attributen und Datensätzen die Ergebnistabelle bestehen soll, kannst du mit den Minus-Schaltflächen (Attribut bzw. Datensatz löschen) und den Plus-Schaltflächen (rechts ein Attribut bzw. unten einen Datensatz ergänzen) verändern. Attributnamen und -werte kannst du reinschreiben.</p><p>Probiere das nun aus und sende eine Ergebnistabelle mit Platz für vier Attribute und fünf Datensätze. Wenn die meisten Attributnamen und -werte leer sind, ist das diesmal okay.</p>",
                locked: true,
                check({ data }) {
                    const colCount = data[0].length;
                    if (colCount != 4)
                        return { msg: 'Es soll 4 und nicht ' + colCount + ' Attribute geben', fine: 1 };
                    const rowCount = data.length - 1;
                    if (rowCount != 5)
                        return { msg: 'Es soll 5 und nicht ' + rowCount + ' Datensätze geben', fine: 1 };
                    if (!data.reduce((prev, row) => (prev || row.reduce((prev, cell) => (prev || cell), false)), false))
                        return { msg: 'Diese Ergebnistabelle darf nicht völlig leer sein', fine: 1 };
                    return { unlock: true };
                }
            },
            {
                task: "<p>Durch Anklicken der Schaltfläche unter der jeweiligen Datenbanktabelle kannst du ihre Daten komplett in den Ergebnistabellenentwurf kopieren. Vor dem Versenden ist eine Bearbeitung natürlich noch möglich, aber so ersparst du dir oft viel Tipparbeit.</p><p>Probiere das ebenfalls aus: Versende den Inhalt der Tabelle <code>STRECKE</code>. Entferne aber vor dem Versand den Datensatz zur Gerade namens <code>a</code>.</p>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name;x1;y1;x2;y2\nb;7;4;7;3\nc;7;4;5;4'
            },
            {
                task: "Jetzt geht's los - hier ist die erste Abfrage. Stelle die Ergebnistabelle richtig zusammen: <code><strong>SELECT</strong> * <strong>FROM</strong> PUNKT</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name;x;y\nA;7;4\nB;5;4\nC;7;3\nP;2;3'
            },
            {
                task: "Nächste Abfrage: <code><strong>SELECT</strong> name <strong>FROM</strong> STRECKE</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name\na\nb\nc'
            },
            {
                task: "Jetzt <code><strong>SELECT</strong> name, y <strong>FROM</strong> PUNKT</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name;y\nA;4\nB;4\nC;3\nP;3'
            },
            {
                task: "Nächste Abfrage: <code><strong>SELECT</strong> x1, x2 <strong>FROM</strong> STRECKE</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'x1;x2\n5;7\n7;7\n7;5'
            },
            {
                task: "Und nun <code><strong>SELECT</strong> x, y, name <strong>FROM</strong> PUNKT</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'x;y;name\n7;4;A\n5;4;B\n7;3;C\n2;3;P'
            },
            {
                task: "Noch eine kurze Abfrage: <code><strong>SELECT</strong> * <strong>FROM</strong> STRECKE</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name;x1;y1;x2;y2\na;5;4;7;3\nb;7;4;7;3\nc;7;4;5;4'
            },
            {
                task: "Das schaffst du auch noch: <code><strong>SELECT</strong> name, x1, x2, y1, y2 <strong>FROM</strong> STRECKE</code>",
                locked: true,
                fine: 1,
                orderRelevant: false,
                result: 'name;x1;x2;y1;y2\na;5;7;4;3\nb;7;7;4;3\nc;7;5;4;4'
            }
        ]
    };
    /*!*/
    return result;
}
