function DbTableDisplay(
    parentNode
) {
    this.node = ScoTools.createHtml('section', parentNode, { 'class': 'DbTableDisplay' });
}

DbTableDisplay.prototype.node = null;
DbTableDisplay.prototype._nodeSelect = null;

DbTableDisplay.prototype._tables = [];
Object.defineProperty(DbTableDisplay.prototype, 'tables', {
    enumerable: true,
    get() { return this._tables.slice(); },
    set(arr) {
        if (!(arr instanceof Array))
            return;
        if (!arr.length) {
            this._tables = [];
            this.node.innerHTML = '';
            this._nodeSelect = null;
            return;
        }
        if (!this.node.firstChild) {
            ScoTools.createHtml('h2', this.node, null, 'Verfügbare Datenbanktabellen');
            this._nodeSelect = ScoTools.createHtml('select', ScoTools.createHtml('label', this.node, null, 'Anzuzeigende Tabelle: '));
            this._nodeSelect.addEventListener('change', () => this.displayTable());
        }
        this._tables = arr.slice();
        this._tables.forEach(({ name }) => ScoTools.createHtml('option', this._nodeSelect, null, name));
        this._nodeSelect.selectedIndex = 0;
        this.displayTable();
        this.showCopy = this.showCopy;
    }
});

/**
* Anzeigen (oder erneut Aufbauen) der ausgewählten Tabelle
* @return undefined
*/
DbTableDisplay.prototype.displayTable = function() {
    const table = ScoTools.createHtml('table', null, { 'class': 'db' });
    const thead = ScoTools.createHtml('thead', table);
    const tbody = ScoTools.createHtml('tbody', table);
    const tableInfos = this._tables[this._nodeSelect.selectedIndex];
    const data = tableInfos ? EditableTable.stringToArray(tableInfos.csv) : [[]];
    data.forEach((row, index) => {
        const tr = ScoTools.createHtml('tr', index ? tbody : thead);
        row.forEach(value => ScoTools.createHtml(index ? 'td' : 'th', tr, null, value));
    });
    const nodeBefore = this._nodeSelect.parentNode;
    if (nodeBefore.nextSibling && (nodeBefore.nextSibling.tagName == 'TABLE'))
        this.node.replaceChild(table, nodeBefore.nextSibling);
    else
        this.node.insertBefore(table, nodeBefore.nextSibling);
}

/** Kopieren-Schaltfläche anzeigen? */
DbTableDisplay.prototype._showCopy = false;
Object.defineProperty(DbTableDisplay.prototype, 'showCopy', {
    enumerable: true,
    get() { return this._showCopy; },
    set(v) {
        this._showCopy = v ? true : false;
        const hasBtn = (this.node.lastChild && (this.node.lastChild.tagName == 'BUTTON'));
        if (!this._showCopy && hasBtn) {
            this.node.removeChild(this.node.lastChild);
            return;
        }
        if (this._showCopy && !hasBtn && this.node.firstChild) {
            const btn = ScoTools.createHtml('button', this.node, { 'class': 'fullwidth', title: 'Daten in Ergebnistabelle kopieren' });
            ScoTools.createHtml('img', btn, { src: 'share/Edit-copy.svg', alt: '📋' });
            ScoTools.createHtml(false, btn, null, ' Daten übernehmen');
            btn.addEventListener('click', () => this.copyActive());
        }
    }
});

/** Aufzurufende Funktion bei Betätigung der Schaltfläche „Daten übernehmen“ */
DbTableDisplay.prototype.oncopy = function(data) { }

DbTableDisplay.prototype.copyActive = function() {
    if (!(typeof this.oncopy == 'function') && !(this.oncopy instanceof Function))
        return;
    this.oncopy(this._tables[this._nodeSelect.selectedIndex]);
}
